# -*- coding: utf-8 -*-

# Copyright (C) 2014-2018 Avencall
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

from sqlalchemy.sql import and_

from xivo_dao.alchemy.agentfeatures import AgentFeatures
from xivo_dao.alchemy.entity import Entity
from xivo_dao.alchemy.userfeatures import UserFeatures
from xivo_dao.alchemy.linefeatures import LineFeatures
from xivo_dao.alchemy.usersip import UserSIP
from xivo_dao.alchemy.voicemail import Voicemail
from xivo_dao.alchemy.user_line import UserLine
from xivo_dao.alchemy.extension import Extension
from xivo_dao.resources.utils.search import (
    SearchSystem, SearchConfig, SearchCombined, SearchFuzzy
)

import logging
logger = logging.getLogger(__name__)


def get_configuration(table=UserFeatures):
    configuration = {
        'table': table,
        'columns': {
            'id': table.id,
            'uuid': table.uuid,
            'firstname': table.firstname,
            'lastname': table.lastname,
            'fullname': (table.firstname + " " + table.lastname),
            'caller_id': table.callerid,
            'description': table.description,
            'userfield': table.userfield,
            'email': table.email,
            'mobile_phone_number': table.mobilephonenumber,
            'music_on_hold': table.musiconhold,
            'outgoing_caller_id': table.outcallerid,
            'preprocess_subroutine': table.preprocess_subroutine,
            'entity': Entity.name,
            'voicemail_number': Voicemail.mailbox,
            'provisioning_code': LineFeatures.provisioning_code,
            'exten': Extension.exten,
            'extension': Extension.exten,
            'context': Extension.context,
            'enabled': table.enabled,
            'agentid': table.agentid
        },
        'search': [
            'fullname',
            'description',
            'userfield',
            'email'
        ],
        'search_weights': {
            'fullname': 5,
            'description': 1,
            'userfield': 1,
            'email': 1
        },
        'default_sort': 'lastname',
        'similarity_threshold': 0.1,
        'unique_column': 'id'
    }
    return configuration


class UserSearchSystem(SearchSystem):

    def search_from_query(self, query, parameters=None):
        query = self._search_on_extension(query)
        term = parameters.get('search')
        logger.error(parameters)
        if not term:
            return self._search_from_query(query, parameters)

        results = SearchCombined(self.config).search_from_query(query, parameters)
        if len(results) == 0:
            results = SearchFuzzy(self.config).search_from_query(query, parameters)
        return results


    @staticmethod
    def _search_on_extension(query, table=UserFeatures):
        return (query
                .outerjoin(Entity,
                           table.entity_id == Entity.id)
                .outerjoin(UserLine,
                           UserLine.user_id == table.id)
                .outerjoin(LineFeatures,
                           and_(LineFeatures.id == UserLine.line_id,
                                LineFeatures.commented == 0))
                .outerjoin(Extension,
                           and_(UserLine.extension_id == Extension.id,
                                Extension.commented == 0))
                .outerjoin(Voicemail,
                           and_(table.voicemailid == Voicemail.uniqueid,
                                Voicemail.commented == 0))
                .outerjoin(AgentFeatures,
                           table.agentid == AgentFeatures.id)
                .outerjoin(UserSIP,
                           and_(LineFeatures.protocol == 'sip',
                                LineFeatures.protocolid == UserSIP.id)))


user_search = UserSearchSystem(SearchConfig(**get_configuration()))
